<?php
/**
 * Registro20SEFIP.
 *
 * @package Logic
 * @author  alessandrobartels
 *
 */
class Registro20SEFIP extends RegistroSEFIP{


    const REGISTRO_20 = '20';

    const CARACTER_HIFEN = ' - ';

    private $_estrutura;
    private $_endereco;

    private $_tpInscricao;
    private $_nrInscricao;
    private $_tpInscricaoTomador;
    private $_nrInscricaoTomador;
    private $_nrRecolhimento;
    private $_fpas;
    private $_valorSalarioFamilia;


    /**
     * Gera Tipo de Registro 20.
     *
     * @param EstruturamEntity $estrutura          Instancia de EstruturamEntity.
     * @param EnderecoEntity   $endereco           Instancia de EnderecoEntity.
     *
     * @return void
     *
     * @author alessandrobartels - 04/01/2012
     */
    public function geraRegistro20(EstruturamEntity $estrutura, EnderecoEntity $endereco){
        $this->_estrutura = $estrutura;
        $this->_endereco = $endereco;

        $this->_nrRecolhimento = $this->getCodigoRecolhimento();
        $this->_fpas = $this->_estrutura->historico->getCdfpas();

        $this->_geraRegistro();
    }


    /**
     * Atribui os dados no atributo da classe Pai.
     *
     * @return void
     *
     * @author alessandrobartels - 04/01/2012
     */
    private function _geraRegistro(){
        // Bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_20, $this->_tpInscricao, $this->_nrInscricao);

        // Bloco 4 | 018 | Tipo de Inscricao do tomador.
        $this->atribuiDado($this->_tpInscricaoTomador, 1);

        // Bloco 5 | 019-032 | Inscricao do tomador.
        $this->atribuiDado($this->_nrInscricaoTomador, 14, parent::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 6 | 033-053 | Preencher com zeros.
        $this->atribuiDado(parent::CARACTER_ZERO, 21, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 7 | 054-093 | Raz�o Social.
        $this->_atribuiRazaoSocial($this->_estrutura->historico->getNmrazsocestrut());

        // Bloco 8 | 094-143 | Endereco.
        $endereco = $this->_endereco->getDsendereco();
        $this->atribuiDado($endereco, 50, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 9 | 144-163 | Bairro.
        $bairro = $this->_endereco->bairro;
        $this->atribuiDado($bairro, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 10 | 164-171 | CEP. @todo Diferentes de 20000000, 30000000, 70000000 ou 80000000.
        $cep = $this->_endereco->getNrcependereco();
        $this->atribuiDado($cep, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 11 | 172-191 | Cidade.
        $cidade = $this->_endereco->municipio;
        $this->atribuiDado($cidade, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 12 | 192-193 | Estado.
        $estado = $this->_endereco->getSgestado();
        $this->atribuiDado($estado, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 13 | 194-197 | Codigo pagamento GPS.
        // @todo Codigo GPS deve vir do parametro de INSS.
        $codigoGPS = 2100;
        $this->_atribuiCodigoGPS($codigoGPS);

        // Bloco 14 | 198-212 | Salario Familia.
        /* @todo Pegar os valores do itemCalculo. Validar para competencia 13 */
        $this->_atribuiSalarioFamilia($this->_valorSalarioFamilia);

        // Bloco 15 | 213-227 | Contrib. Desc. Empregado Referente Compet. 13 (preencher com 0).
        $this->atribuiDado(parent::CARACTER_VAZIO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 16 | 228-228 | Indicador de Valor Negativo / Positivo (preencher com 0).
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 17 | 229-242 | Valor devido a Previdencia referente compet. 13 (preencher com 0).
        $this->atribuiDado(parent::CARACTER_VAZIO, 14, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 18 | 243-257 | Valor de retencao.
        // @todo Verificar origem do parametro de retencao.
        $vrRetencao = parent::CARACTER_VAZIO;
        $this->_atribuiValorRetencao($vrRetencao);

        // Bloco 19 | 258-272 | Valor das faturas emitidas para o tomador.
        $valor = parent::CARACTER_VAZIO;
        $this->_atribuiValorFaturaTomador($valor);

        // Bloco 20 | 273-317 | Zeros.
        $this->atribuiDado(parent::CARACTER_VAZIO, 45, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 21 | 318-359 | Brancos.
        $this->atribuiDado(parent::CARACTER_VAZIO, 42, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);

        // Bloco 22 | 360 | Final de linha. Inserir '*'.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ASTERISCO);
    }


    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }


    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }


    public function setNrInscricaoTomador($nrInscricaoTomador)
    {
        $this->_nrInscricaoTomador = $nrInscricaoTomador;
    }


    public function setTpInscricaoTomador($tpInscricaoTomador)
    {
        $this->_tpInscricaoTomador = $tpInscricaoTomador;
    }


    /**
     * Atribui codigo da GPS.
     *
     * @param string $codigoGPS Codigo da GPS.
     *
     * @return void
     *
     * @author thalesvalias - 31/01/2012
     *         felipesantos - 07/05/2015
     */
    private function _atribuiCodigoGPS($codigoGPS){

        $codRecol = array(
            '211', '317', '337'
        );

        if (array_search($this->_nrRecolhimento, $codRecol) >= 0) {
            $codigoGPS = parent::CARACTER_BRANCO;
        }
        else {
            if (is_null($this->getCodPagtoGPS())) {
//              se for optante pelo simples, define o codigo 2003
                if ($this->_simples === self::SIMPLES_OPTANTE) {
                    $codigoGPS = '2003';
                }

//              caso seja o empregador doméstico, a gps deve ser gerada com o código 2216
                if ($this->_idTipoEmpr === 5) {
                    $codigoGPS = '2216';
                }

//              caso seja o empregador rural, a gps deve ser gerada com o código 2216
                if ($this->_idTipoEmpr === 6) {
                    $codigoGPS = '2208';
                }
            } else {
                $codRecol = array(
                    '130', '135', '155', '608',
                );

                if (array_search($this->_nrRecolhimento, $codRecol) === false) {
                    $codigoGPS = parent::CARACTER_BRANCO;
                }
                else {
                    $codigoGPS = $this->getCodPagtoGPS();
                }
            }
        }

        $this->atribuiDado($codigoGPS, 4, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    private function _atribuiRazaoSocial($razaosocial){
        $razaosocialtradado = str_replace(self::CARACTER_HIFEN, parent::CARACTER_BRANCO, $razaosocial);
        $this->atribuiDado($razaosocialtradado, 40, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    /**
     * Atribui valor do Salario Familia.
     *
     * @param float $vrSalarioFamilia Valor do salario familia.
     *
     * @return void
     *
     * @author thalesvalias - 31/01/2012
     */
    private function _atribuiSalarioFamilia($vrSalarioFamilia) {

        $codRecol = array(
            '150', '155', '608',
        );

        // @todo Verificar leiaute - validar para a competencia 13.
        if ((array_search($this->_nrRecolhimento, $codRecol) === false) || ($this->_fpas === '868')) {
            $vrSalarioFamilia = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($vrSalarioFamilia, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    /**
     * Atribui valor de retencao.
     *
     * @param float $vrRetencao Valor de retencao.
     *
     * @return void
     *
     * @author thalesvalias - 31/01/2012
     */
    private function _atribuiValorRetencao($vrRetencao) {

        $codRecol = array(
            '150', '155',
        );

        if (array_search($this->_nrRecolhimento, $codRecol) === false) {
            $vrRetencao = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($vrRetencao, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    /**
     * Atribui valor das faturas emitidas para o tomador.
     *
     * @param float $valor Valor das faturas emitidas para o tomador.
     *
     * @return void
     *
     * @author thalesvalias - 31/01/2012
     */
    private function _atribuiValorFaturaTomador($valor) {

        if ($this->_nrRecolhimento !== '211') {
            $valor = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($valor, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    public function alteraDadoRegistro20($registroMag, $dado) {

        // Salario Familia - Posicao Inicial = 198 Tamanho = 15. Subtrai 1 da posicao inicial, pois o array inicia em 0.
        $linha = $this->alteraDado($registroMag->getDslinharegistro(), $dado[self::SEFIP_SALARIO_FAMILIA], (198 - 1), 15,
            parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        $registroMag->setDslinharegistro($linha);
        $registroMag->setVrsalfamilia($dado[self::SEFIP_SALARIO_FAMILIA]);
        $this->_valorSalarioFamilia = $registroMag->getVrsalfamilia();

    }

}